import os
import re
import sys
import subprocess

def pop_left_slice(inputlist, length):
    outputlist = inputlist[0:length]
    del inputlist[:length]
    return outputlist

def f_vector(outlist):
    x,y,z = pop_left_slice(line,3)
    outlist.append(f"<{x},{y},{z}>,")

def f_value(outlist):
    value = pop_left_slice(line,1)[0]
    outlist.append(value+",")

def f_number(outlist):
    number = pop_left_slice(line,1)[0]
    outlist.append(number+",")
    return int(number)

def f_value_array(outlist, length):
    outlist.append(f"  //label: {lbl}\n  array[{length}]"+"{"+",".join(pop_left_slice(line,length))+"},\n")

def mx_value_array(outlist, length):
    m=pop_left_slice(line,length)
    mx = max([int(i) for i in m])
    outlist.append(f"  //label: {lbl}\n  array[{length}]"+"{"+",".join(m)+"},\n")
    return mx

def f_vector_array(outlist, length):
    rv = pop_left_slice(line, length)
    rv = [f'<{i[1:-1]}>' for i in rv]
    rv = ",".join(rv)
    outlist.append(f"  //label: {lbl}\n  array[{length}]"+"{\n    "+rv+"\n  }\n")
    
def f_nested_value_array(outlist, length):
    fi = pop_left_slice(line,length)
    fi = [f'{i[1:-1]}' for i in fi]
    fi = [i.split(",") for i in fi]
    fi = [f"    array[{len(i)}]"+"{"+",".join(i)+"},\n" for i in fi]
    outlist.append(f"  //label: {lbl}\n  array[{length}]"+"{\n"+"".join(fi)+"  },\n")

def arrange_order(order, preferred_order):
    """
    De- and reconstruct the output control string of the command line argument.
    Remove doubles, require %i, the lables. Use the full 3D vector %q instead of
    separate x,y,z as these can easily be extracted from a vector in POV-Ray. To
    make sure all parses well, some tests are done for dependencies in the data.
    For this also the order of the string is adjusted.
    """
    order = list(set(order))
    if not "%i" in order:
        order.append("%i")
    if True in [True for i in ["%x","%y","%z"] if i in order]:
        order = [i for i in order if i not in ["%x","%y","%z"]]
        if not "%q" in order:
            order.append("%q")
    if True in [True for i in ["%p","%P","%o"] if i in order]:
        if not "%w" in order:
            order.append("%w")
    if True in [True for i in ["%e","%a","%f","%t","%l","%n"] if i in order]:
        if not "%s" in order:
            order.append("%s")
    if "%A" in order:
        if not "%a" in order:
            order.append("%a")
    order.sort(key=lambda i: preferred_order.index(i))
    return order

preferred_order = [
    '%i', '%w', '%s', '%a', '%A', '%e', '%F', '%f', '%t', '%l',
    '%n', '%p', '%P', '%o', '%q', '%r', '%m', '%g', '%E', '%v',
    '%c', '%C'
]
write_order =[
    '%i', '%q', '%c', '%C', '%r', '%v', '%m', '%s', '%F', '%g',
    '%E', '%w', '%t', '%P', '%p', '%l', '%n', '%e', '%f', '%o',
    '%a', '%A'
]
function = {
    "%i" : f_number,       "%w" : f_number,       "%s" : f_number,
    "%a" : mx_value_array, "%q" : f_vector,       "%r" : f_value,
    "%p" : f_vector_array, "%P" : f_vector_array, "%o" : f_value_array,
    "%m" : f_value,        "%g" : f_number,       "%E" : f_value,
    "%e" : f_value_array,  "%F" : f_value,        "%A" : f_value_array,
    "%f" : f_value_array,  "%l" : f_vector_array, "%n" : f_value_array,
    "%v" : f_value,        "%c" : f_vector,       "%C" : f_vector,
    "%t" : f_nested_value_array
} 

command = sys.argv[1:]
if not "-o" in command:
    command.insert(0, "-o")
try:
    c_idx = command.index("-c")
    order = command[c_idx+1]
except ValueError:
    print("no -c option defined,\nadding it for default output\n")
    order = "%i, %q, %v"
    command.insert(0,order)
    command.insert(0,"-c")
    c_idx = 0
filename = command[-1]
order = re.findall("%[a-z]",order,re.M|re.I)
order = arrange_order(order, preferred_order)
command[c_idx+1]=f"{' '.join(order)}"
command.insert(0, "voro++")
return_val = subprocess.run(command)
content = {}
for i in order:
    content[i]=[]
inputfile = filename+".vol"
for points, line in enumerate(open(inputfile,'r'), 1):        
    line = line.strip()
    line = line.split(" ")
    for action in order:
        if action == "%i":
            lbl = function[action](content[action])
        elif action == "%w":
            v = function[action](content[action])
        elif action == "%s":
            f = function[action](content[action])
        elif action == "%a":
            mx = function[action](content[action],f)
        elif action in ["%p", "%P", "%o"]:
            function[action](content[action],v)
        elif action in ["%e", "%f", "%t", "%l", "%n"]:
            function[action](content[action],f)
        elif action == "%A":
            function[action](content[action],mx+1)
        else:
            function[action](content[action])

headers = {
    "%i" : f"// %i\n#declare Labels = array[{points}]"+"{",
    "%q" : f"// %q\n#declare Points = array[{points}]"+"{",
    "%r" : f"// %r\n#declare Radii = array[{points}]"+"{",
    "%w" : f"// %w\n#declare NumVertices = array[{points}]"+"{",
    "%p" : f"// %p\n#declare RelVertices = array[{points}]"+"{\n",
    "%P" : f"// %P\n#declare GlobalVertices = array[{points}]"+"{\n",
    "%o" : f"// %o\n#declare VertexOrders = array[{points}]"+"{\n",
    "%m" : f"// %m\n#declare MaxRadius = array[{points}]"+"{",
    "%g" : f"// %g\n#declare NumEdges = array[{points}]"+"{",
    "%E" : f"// %E\n#declare EdgeDistance = array[{points}]"+"{",
    "%s" : f"// %s\n#declare NumFaces = array[{points}]"+"{",
    "%e" : f"// %e\n#declare FacePerimeter = array[{points}]"+"{\n",
    "%F" : f"// %F\n#declare SurfaceArea = array[{points}]"+"{",
    "%a" : f"// %a\n#declare FreqFaces = array[{points}]"+"{\n",
    "%A" : f"// %A\n#declare FacesOrders = array[{points}]"+"{\n",
    "%f" : f"// %f\n#declare FaceArea = array[{points}]"+"{\n",
    "%t" : f"// %t\n#declare FaceVerticesIndex = array[{points}]"+"{\n",
    "%l" : f"// %l\n#declare FaceNormal = array[{points}]"+"{\n",
    "%n" : f"// %n\n#declare Neighbours = array[{points}]"+"{\n",
    "%v" : f"// %v\n#declare Volume = array[{points}]"+"{",
    "%c" : f"// %c\n#declare RelCentroid = array[{points}]"+"{",
    "%C" : f"// %C\n#declare GlobalCentroid = array[{points}]"+"{",
}

order.sort(key=lambda i: write_order.index(i))
filename = os.path.splitext(filename)[0]+".inc"
with open(filename, 'w') as outfile:
    for i in order:
        outfile.write(headers[i]+"".join(content[i])+"};\n")
print(f"{filename}")

    
