#version 3.7;
global_settings{ assumed_gamma 1.0 }
#include "transforms.inc"

#default{ finish{ ambient 0.1 diffuse 0.9 }} 

camera {
  perspective angle 10
  location  <0 , 20 ,-10>
  look_at   <0,0,0>
}
light_source{< 500,3000,-500> rgb 1}
light_source{<-3000,3000,-3000> rgb .5}



#macro VoroPlanes(Normals,FaceIndex,Vertices)
  /*==
  Creates a single voronoi cell from data generated by Voro++ by intersecting
  planes
  
  Normals   (arr, vec) : an array of normals
  FaceIndex (arr, int) : an array with indices that points to vertices
  Vertices  (arr, vec) : an array with vertices of the cell faces
  */
  #local Length = dimension_size(Normals,1);
  #local Voro = intersection{
    #for(I,0,Length-1,1)
      #local Normal = Normals[I];
      #local Face = FaceIndex[I];
      #local VertexIndex = Face[0]; // only need a single vertex on the plane
      #local Vertex = Vertices[VertexIndex];
      plane{Normal,vdot(Normal,Vertex)}    
    #end
  } 
  object{Voro}
#end

#macro VoroPolygons(Normals, FaceIndex, Vertices)
  /*==
  Creates a single voronoi cell from data generated by Voro++ by constructing
  a polygon per face.
  
  Normals   (arr, vec) : an array of normals
  FaceIndex (arr, int) : an array with indices that points to vertices
  Vertices  (arr, vec) : an array with vertices of the cell faces
  */
  #local Length = dimension_size(FaceIndex,1);
  #local Voro = union{
    #for(Face,0,Length-1,1)
      #local VertexIndices = FaceIndex[Face];
      #local Normal = Normals[Face];
      #local Distance = vdot(Normal,Vertices[VertexIndices[0]]);
      #local Transform = transform{translate Normal*Distance}; 
      #local VILength = dimension_size(VertexIndices,1);
      polygon{
        VILength+1,
        #for(VertexIndex,0,VILength-1,1)
          #local Index = VertexIndices[VertexIndex];
          #local Vertex = Vertices[Index];
          #local Vertex = VProject_Plane(Vertex, Normal);
          #local Vertex = vtransform(Vertex, Transform);
          #if (VertexIndex = 0)
            #local P0 = Vertex;
          #end 
          Vertex
        #end
        P0
      }
    #end
  }
  object{Voro}
#end

#macro VoroVertex(FaceIndex, Vertices)
  #local Length = dimension_size(FaceIndex,1);
  #local Voro = union{
    #for(Face,0,Length-1,1)
      #local VertexIndices = FaceIndex[Face];
      #local VILength = dimension_size(VertexIndices,1);
        #for(VertexIndex,0,VILength-1,1)
          #local Index = VertexIndices[VertexIndex];
          #local Vertex = Vertices[Index];
          //#debug concat(vstr(3,Vertex,",",6,6),"\n")
          sphere{Vertex,0.02 pigment{rgb x}}
        #end
        //#debug "\n"
    #end
  }
  object{Voro}
#end

#include "vorosphere.inc"

#declare FaceIndex = FaceVerticesIndex[0];
#declare Vertices  = GlobalVertices[0];
#declare Normals   = FaceNormal[0];
object{VoroPlanes(  Normals, FaceIndex, Vertices) pigment{rgb 1} translate <1,0,0>}
object{VoroPolygons(Normals, FaceIndex, Vertices) pigment{rgb 1} translate <-1,0,0>}
object{VoroVertex(FaceIndex, Vertices) pigment{rgb 1} translate <1,0,0>}
object{VoroVertex(FaceIndex, Vertices) pigment{rgb 1} translate <-1,0,0>}

sphere{0,0.1 pigment{rgb z}}
